/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */

#ifndef DEBUG_H
#define DEBUG_H

#include <linux/kernel.h>
#include <linux/time.h>

#ifdef __cplusplus
extern "C" {
#endif

extern int debug_on;

typedef struct iodstat iodstat, *iodstat_p;

struct iodstat {
	struct timeval st_wait;
	struct timeval st_serv;
	struct timeval end_serv;
	int	 type;
	int	 numaccess;
	int	 fd;
	int 	 offset; 
};


#ifdef DEBUG

#ifdef __GNUC__
#define LOG(x...) do { if (debug_on > 0) { printk(x); } } while(0)
#else 
static void LOG(char * fmt, ...) 
{
  va_list ap;
  /* FIXME: Can error/log messages be > than this. kernel stack pages
  are notoriously small on many machines */
  char str[256];
  va_start(ap, fmt);

  if (debug_on <= 0) return;

  vsprintf(str, fmt, ap);
  printk("%s\n",str);
  return;
};
#endif /* __GNUC__ */
#define LOG1(x,y) if (debug_on>0) { printk(x, (y));  }
#define LOG2(x,y,z) if (debug_on>0) { printk(x, (y), (z));  }
#define LOG3(w,x,y,z) if (debug_on>0) { printk(w, (x), (y), (z)); }
#define LOG4(v,w,x,y,z) if (debug_on>0) { printk(v,(w),(x),(y),(z));  }

#else /* DEBUG */

#ifdef __GNUC__
#define LOG(x...);
#else
static void LOG(char * fmt, ...) { ; };
#endif
#define LOG1(x,y);
#define LOG2(x,y,z);
#define LOG3(w,x,y,z);
#define LOG4(v,w,x,y,z);
#endif /* DEBUG */

#if 1

#ifdef __GNUC__

#define ERR(x...) do { printk(x); }  while(0)
#else 
static void ERR(char * fmt, ...) 
{
  char str[256];	
  va_list ap;
  va_start(ap, fmt);

  vsprintf(str, fmt, ap);
  printk("%s\n",str);
  return;
};
#endif /* __GNUC__ */

#define ERR1(x,y) { printk(x, (y)); }
#define ERR2(x,y,z) { printk(x, (y), (z)); }
#define ERR3(w,x,y,z) { printk(w, (x), (y), (z)); }
#else
#ifdef __GNUC__
#else
static void ERR(char * fmt, ...) { ; };
#endif
#define ERR(x...);
#define ERR1(x,y);
#define ERR2(x,y,z);
#define ERR3(w,x,y,z);
#endif

#ifdef __cplusplus
}
#endif

#endif
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
