/*
 * (C) 1995-2004 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */

/*************************************
 * Phil Carns    Nov 99   Parl Lab
 * Test code for pvfsd kernel communication
 * This one writes a request into the device file
 ***************************************/

/* these three are included to make the types in ll_pvfs.h work */
#include <stdint.h>
#include <inttypes.h>
#include <time.h>

#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "../ll_pvfs.h"

#define TEST_LOOP 666
#define TEST_UPWAIT 667
#define TEST_UPWAIT_COMM 668
#define TEST_RD_XFER 670
#define PHASE_TWO_WRITE 671
#define TEST_WR_XFER 672

int main(int argc, char **argv){

	struct pvfs_upcall *putreq = NULL;
	struct pvfs_downcall bounce_down;
	int devfile = -1;
	int testfile = -1;
	int req_sz = sizeof(struct pvfs_upcall);
	int read_sz = 0;
	fd_set readfds;
	struct timeval tv;
	char testbuf[] = "Yow!\n";
	char *buffer = NULL;

	if(argc != 1){
		printf("dumbass.\n");
		exit(-1);
	}

	putreq = (struct pvfs_upcall*)malloc(sizeof(struct pvfs_upcall));
	if(!putreq){
		perror("malloc");
		exit(-1);
	}

	devfile = open("/dev/pvfsd", O_RDWR);
	if(devfile < 0){
		perror("open");
		exit(-1);
	}

	/* we want to select on the device before trying to read it... */
	FD_ZERO(&readfds);
	FD_SET(devfile, &readfds);
	tv.tv_sec = 20;
	tv.tv_usec = 0;
	if((select((devfile+1), &readfds, NULL, NULL, &tv)) > 0){
		if(FD_ISSET(devfile, &readfds)){
			read_sz = read(devfile, putreq, req_sz);
			if((read_sz!=0)&&(read_sz<req_sz)){
				perror("read");
				close(devfile);
				exit(-1);
			}
			else if(read_sz==0){
				printf("No data available.\n");
				close(devfile);
				return(0);
			}

			if(putreq->magic != PVFS_UPCALL_MAGIC){
				printf("Bad magic.\n");
			}
			printf("Recieved sequence number: %d\n", putreq->seq);

			if(putreq->type == TEST_RD_XFER){
				/* we need to make a buffer here to transfer data from */
				printf("Recieved xfer read test request.\n");

				buffer = (char*)malloc((size_t)putreq->xfer.size);
				if(!buffer){
					perror("malloc");
					exit(-1);
				}
				
				if((testfile = open("testin.txt", O_RDONLY)) == -1){
					perror("open");
					exit(-1);
				}
				if((read(testfile, buffer, putreq->xfer.size)) !=
					putreq->xfer.size){
					perror("read");
					exit(-1);
				}
				close(testfile);
				bounce_down.seq = putreq->seq;
				bounce_down.magic = PVFS_DOWNCALL_MAGIC;
				bounce_down.type = TEST_RD_XFER;
				printf("buffer to read from: %lx\n", (long)buffer);
				bounce_down.xfer.ptr = buffer;
				bounce_down.xfer.size = putreq->xfer.size;

			}
			else if(putreq->type == TEST_WR_XFER){
				/* make a buffer here to transfer to. */
				printf("Recieved xfer write test request.\n");
				printf("Buffer before request: %s\n", testbuf);
	
				bounce_down.seq = putreq->seq;
				bounce_down.magic = PVFS_DOWNCALL_MAGIC;
				bounce_down.type = WRITE_OP;
				bounce_down.xfer.ptr = testbuf;
				bounce_down.xfer.size = strlen(testbuf);
		
				printf("Sending first reply...\n");
				if((write(devfile, &bounce_down, sizeof(struct pvfs_downcall)))!=
					sizeof(struct pvfs_downcall)){
					 perror("write");
				}
		
				printf("Buffer after request: %s\n", testbuf);
				bounce_down.type = TEST_WR_XFER;

			}
			else{
				bounce_down.seq = putreq->seq;
				bounce_down.magic = PVFS_DOWNCALL_MAGIC;
				bounce_down.type = TEST_UPWAIT_COMM;
			}
			
			printf("Sending it back...\n");
			if((write(devfile, &bounce_down, sizeof(struct pvfs_downcall)))!=
				sizeof(struct pvfs_downcall)){
			    perror("write");
		   }
			printf("Reply sent back.\n");

		}
	}

	close(devfile);

	return(0);
}


/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
