/*
 * (C) 1995-2004 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */

/*************************************
 * Phil Carns    Nov 99   Parl Lab
 * Rob Ross
 * test harness for handling requests that have come from
 * application to vfs to ll_pvfs to pvfsdev up to here and replying
 * to them in user space without actually talking to a filesystem
 ***************************************/

/* these three are included to make the types in ll_pvfs.h work */
#include <stdint.h>
#include <inttypes.h>
#include <time.h>

#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "../ll_pvfs.h"

int handle_request(struct pvfs_upcall* incoming, struct
	pvfs_downcall* outgoing);


int main(int argc, char **argv){

	struct pvfs_upcall *incoming = NULL;
	struct pvfs_downcall *outgoing = NULL;
	int devfile = -1;
	int req_sz = sizeof(struct pvfs_upcall);
	int out_sz = sizeof(struct pvfs_downcall);
	int read_sz = 0;
	fd_set readfds;

	if(argc != 1){
		printf("dumbass.\n");
		exit(-1);
	}

	incoming = (struct pvfs_upcall*)malloc(req_sz);
	if(!incoming){
		perror("malloc");
		exit(-1);
	}
	outgoing = (struct pvfs_downcall*)malloc(out_sz);
	if(!outgoing){
		perror("malloc");
		exit(-1);
	}

	devfile = open("/dev/pvfsd", O_RDWR);
	if(devfile < 0){
		perror("open");
		exit(-1);
	}

	while(1){

		/* we want to select on the device and sit here until it talks to
		 * us */
		FD_ZERO(&readfds);
		FD_SET(devfile, &readfds);
		if((select((devfile+1), &readfds, NULL, NULL, NULL)) > 0){
			if(FD_ISSET(devfile, &readfds)){
				read_sz = read(devfile, incoming, req_sz);
				if(read_sz==0){
					printf("No data available?\n");
					break;
				}
				else if(read_sz<req_sz){
					perror("read");
					break;
				}

				if(incoming->magic != PVFS_UPCALL_MAGIC){
					printf("Bad magic.\n");
					break;
				}
				printf("Recieved sequence number: %d\n", incoming->seq);

				if(handle_request(incoming, outgoing) < 0){
					perror("could not handle request.\n");
					break;
				}

				if((write(devfile, outgoing, out_sz)) != out_sz){
					perror("write");
					break;
				}

			}
		}

	}

	close(devfile);

	return(0);
}


int handle_request(struct pvfs_upcall* incoming, struct
	pvfs_downcall* outgoing)
{

	/* I'm putting big stoopid brackets in here to get around name
	 * clashes as I cut and paste from Rob's code :) */

	/* for lookup op */
	pvfs_char_t *name;
	pvfs_handle_t *handle;
	struct pvfs_meta *meta;
	/* for getmeta */
	struct pvfs_meta root = { 1, 0, S_IFDIR | 0666, 0, 100, 666,
		123123, 123123, 123123, 65536, 1};
	struct pvfs_meta foo = { 2, 0, S_IFREG | 0755, 0, 100, 666,
		123123, 123123, 123123, 65536, 1};
	struct pvfs_meta bar = { 3, 0, S_IFREG | 0755, 0, 100, 42,
		456456, 456456, 456456, 65536, 1};
	/* generic settings */
	memset(outgoing, 0, sizeof(*outgoing));
	outgoing->seq = incoming->seq;
	outgoing->type = incoming->type;
	outgoing->magic = PVFS_DOWNCALL_MAGIC;

	switch(incoming->type){
		case(LOOKUP_OP):
			printf("lookup operation.\n");
			fflush(NULL);
			sleep(1);
			meta = &(outgoing->u.lookup.meta);
			name = incoming->u.lookup.name;
			if (!strcmp(name, "foo:3000/pvfs")) { *meta = root; return 0; }
			if (!strcmp(name, "foo:3000/pvfs/foo")) { *meta = foo; return 0; }
			if (!strcmp(name, "foo:3000/pvfs/bar")) { *meta = bar; return 0; }
			outgoing->error = -EINVAL;
			return 0;
			break;
		case(GETMETA_OP):
			{
				printf("getmeta operation.\n");
				fflush(NULL);
				sleep(1);
				handle = &(incoming->u.getmeta.handle);
				meta = &(outgoing->u.getmeta.meta);
				if (*handle == 1) { *meta = root; return 0; }
				if (*handle == 2) { *meta = foo; return 0; }
				if (*handle == 3) { *meta = bar; return 0; }
				outgoing->error = -ENOENT;
				return 0;
			}
			break;
		case(GETDENTS_OP):
			{
				struct pvfs_dirent dot = { 1, 0, sizeof(struct pvfs_dirent), 
					"."};
				struct pvfs_dirent foo = { 2, sizeof(struct pvfs_dirent),
					sizeof(struct pvfs_dirent), "foo"};
				struct pvfs_dirent bar = { 3, 2*sizeof(struct pvfs_dirent),
					sizeof(struct pvfs_dirent), "bar"};
				struct pvfs_dirent dotdot = { 1, 3*sizeof(struct pvfs_dirent),
					sizeof(struct pvfs_dirent), ".."};
				pvfs_off_t off = 0;
				struct pvfs_dirent dirent;

				printf("getdents operation.\n");
				off = incoming->u.getdents.off;

				outgoing->u.getdents.eof = 0;
				outgoing->u.getdents.count = 1;
				outgoing->xfer.ptr = &dirent;
				outgoing->xfer.size = sizeof(struct pvfs_dirent);

				if (off == 0) { dirent = dot; return 0; }
				if (off == sizeof(struct pvfs_dirent)) { 
					dirent = foo; return 0; }
				if (off == 2*sizeof(struct pvfs_dirent)) { 
					dirent = bar; return 0; }
				if (off == 3*sizeof(struct pvfs_dirent)) { 
					dirent = dotdot; return 0; }
				if(off > 3*sizeof(struct pvfs_dirent)) {
					printf("offset greater than 3.\n");
				}
				printf("offset = %lx\n", (long unsigned int)off);
				/* hey, um zero out the dirent */
				outgoing->error = -ENOENT;
				outgoing->u.getdents.eof = 1;
				outgoing->u.getdents.count = 0;
				return 0;
			}
			break;
		case(STATFS_OP):
			{
				struct pvfs_statfs fake = { 65536, 1048576, 1048575,
					1024, 1024 };
				printf("statfs operation.\n");
				fflush(NULL);
				sleep(1);
				outgoing->u.statfs.statfs = fake;
				return 0;
			}	
			break;
		case(READ_OP):
			{
				pvfs_char_t foo_string[] = "This is the data in file foo.\n";
				pvfs_off_t *offp = &(incoming->u.rw.io.u.contig.off);
				pvfs_size_t len = incoming->u.rw.io.u.contig.size;

				if(len > strlen(foo_string)){
					len = strlen(foo_string);
				}

				printf("read operation.\n");
				if (incoming->u.rw.handle == (long) 2 && *offp == (long) 0) {
					printf("doing the copy thing, %ld bytes\n", (long) len);
					outgoing->xfer.ptr = foo_string;
					outgoing->xfer.size = len;
					outgoing->u.rw.size = len;
					outgoing->u.rw.eof = 0;
					return 0;
				}
				else if (incoming->u.rw.handle == (long) 2 && *offp >=
					(long)len){
					outgoing->xfer.ptr = foo_string;
					outgoing->xfer.size = 0;
					outgoing->u.rw.size = 0;
					outgoing->u.rw.eof = 1;
					return 0;
				}
				else{
					outgoing->error = -ENOENT;
					return 0;
				}

			}
			break;		
		default:
			printf("I don't know how to handle this request...\n");
			outgoing->error = -ENOENT;
			return(0);

	}


	return(0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
